@@ -1,37 +1,43 @@
 """Python 3 API wrapper for FireServiceRota and BrandweerRooster."""
-from typing import Optional
-import json
-import time
 from collections import deque
-import oauthlib.oauth2
-import websocket
-import requests
-from threading import Thread
 import datetime
+import json
 import logging
-
+import threading
+import time
 from typing import Optional
+
+import oauthlib.oauth2
+import requests
 from requests.exceptions import HTTPError, RequestException, Timeout
+import websocket
 
 from .const import (
     FSR_DEFAULT_TIMEOUT,
-    FSR_ENDPOINT_TOKEN,
-    FSR_ENDPOINT_USER,
-    FSR_ENDPOINT_MEMBERSHIPS,
-    FSR_ENDPOINT_SKILLS,
-    FSR_ENDPOINT_DUTY_STANDBY_FUNCTIONS,
     FSR_ENDPOINT_DUTY_STANDBY_FUNCTION,
+    FSR_ENDPOINT_DUTY_STANDBY_FUNCTIONS,
     FSR_ENDPOINT_INCIDENT_RESPONSES,
     FSR_ENDPOINT_INCIDENTS,
+    FSR_ENDPOINT_MEMBERSHIPS,
+    FSR_ENDPOINT_SKILLS,
+    FSR_ENDPOINT_TOKEN,
+    FSR_ENDPOINT_USER,
 )
-from .errors import FireServiceRotaError, InvalidAuthError, ExpiredTokenError, InvalidTokenError
+from .errors import ExpiredTokenError, InvalidAuthError, InvalidTokenError
 
 _LOGGER = logging.getLogger("pyfireservicerota")
 
+
 class FireServiceRota(object):
     """Class for communicating with the fireservicerota API."""
 
-    def __init__(self, base_url = None, username: str = None, password: str = None, token_info: dict = None):
+    def __init__(
+        self,
+        base_url=None,
+        username: str = None,
+        password: str = None,
+        token_info: dict = None,
+    ):
         """Init module"""
         self._base_url = base_url
         self._username = username
@@ -44,14 +50,22 @@ class FireServiceRota(object):
 
         oauth_client = oauthlib.oauth2.LegacyApplicationClient(client_id=None)
         request_body = oauth_client.prepare_request_body(
-                 username=self._username, password=self._password)
-        response = self._request('POST', endpoint=FSR_ENDPOINT_TOKEN, log_msg_action='request tokens',
-                 params=str.encode(request_body), auth_request=True)
+            username=self._username, password=self._password
+        )
+        response = self._request(
+            "POST",
+            endpoint=FSR_ENDPOINT_TOKEN,
+            log_msg_action="request tokens",
+            params=str.encode(request_body),
+            auth_request=True,
+        )
 
         try:
             self._token_info = response
-            _LOGGER.debug(f"Obtained tokens: access {self._token_info['access_token']}, "
-                          f"refresh {self._token_info['refresh_token']}")
+            _LOGGER.debug(
+                f"Obtained tokens: access {self._token_info['access_token']}, "
+                f"refresh {self._token_info['refresh_token']}"
+            )
             return self._token_info
         except (KeyError, TypeError) as err:
             _LOGGER.debug(f"Error obtaining tokens: {err}")
@@ -61,18 +75,26 @@ class FireServiceRota(object):
         """Refresh existing API tokens."""
 
         if not self._token_info:
-           return
+            return
 
         oauth_client = oauthlib.oauth2.LegacyApplicationClient(client_id=None)
         request_body = oauth_client.prepare_refresh_body(
-                 refresh_token=self._token_info['refresh_token'])
-        response = self._request('POST', endpoint=FSR_ENDPOINT_TOKEN, log_msg_action='refresh tokens',
-                 params=str.encode(request_body), auth_request=True)
+            refresh_token=self._token_info["refresh_token"]
+        )
+        response = self._request(
+            "POST",
+            endpoint=FSR_ENDPOINT_TOKEN,
+            log_msg_action="refresh tokens",
+            params=str.encode(request_body),
+            auth_request=True,
+        )
 
         try:
             self._token_info = response
-            _LOGGER.debug(f"Refreshed tokens: access {self._token_info['access_token']}, "
-                          f"refresh {self._token_info['refresh_token']}")
+            _LOGGER.debug(
+                f"Refreshed tokens: access {self._token_info['access_token']}, "
+                f"refresh {self._token_info['refresh_token']}"
+            )
             return self._token_info
         except (KeyError, TypeError) as err:
             _LOGGER.debug(f"Error refreshing tokens: {err}")
@@ -81,8 +103,12 @@ class FireServiceRota(object):
     def _get_userid(self):
         """Get user data."""
 
-        self._user = self._request('GET', endpoint=FSR_ENDPOINT_USER, log_msg_action='get userid',
-                 auth_request=False)
+        self._user = self._request(
+            "GET",
+            endpoint=FSR_ENDPOINT_USER,
+            log_msg_action="get userid",
+            auth_request=False,
+        )
 
         _LOGGER.debug(f"Userid data: {self._user}")
 
@@ -93,20 +119,29 @@ class FireServiceRota(object):
             self._get_userid()
 
         today = datetime.datetime.now().astimezone()
-        tomorrow = today + datetime.timedelta(days = 1)
-        id =  self._user['memberships'][0]['id']
-        url = FSR_ENDPOINT_MEMBERSHIPS.format(id, today.strftime("%Y-%m-%dT00:00:00%z"), tomorrow.strftime("%Y-%m-%dT00:00:00%z"))
+        tomorrow = today + datetime.timedelta(days=1)
+        id = self._user["memberships"][0]["id"]
+        url = FSR_ENDPOINT_MEMBERSHIPS.format(
+            id,
+            today.strftime("%Y-%m-%dT00:00:00%z"),
+            tomorrow.strftime("%Y-%m-%dT00:00:00%z"),
+        )
 
-        response = self._request('GET', endpoint=url, log_msg_action='get memberships',
-                 auth_request=False)
+        response = self._request(
+            "GET", endpoint=url, log_msg_action="get memberships", auth_request=False
+        )
 
         return response
 
     def get_skills(self):
         """Get skills."""
 
-        response = self._request('GET', endpoint=FSR_ENDPOINT_SKILLS, log_msg_action='get skills',
-                 auth_request=False)
+        response = self._request(
+            "GET",
+            endpoint=FSR_ENDPOINT_SKILLS,
+            log_msg_action="get skills",
+            auth_request=False,
+        )
 
         return response
 
@@ -115,8 +150,12 @@ class FireServiceRota(object):
 
         url = FSR_ENDPOINT_DUTY_STANDBY_FUNCTION.format(id)
 
-        response = self._request('GET', endpoint=url, log_msg_action='get standby function',
-                 auth_request=False)
+        response = self._request(
+            "GET",
+            endpoint=url,
+            log_msg_action="get standby function",
+            auth_request=False,
+        )
 
         return response
 
@@ -126,22 +165,36 @@ class FireServiceRota(object):
         url = FSR_ENDPOINT_INCIDENT_RESPONSES.format(id)
 
         if status:
-            json = {'status': 'acknowledged'}
+            json = {"status": "acknowledged"}
         else:
-            json = {'status': 'rejected'}
+            json = {"status": "rejected"}
+
+        self._request(
+            "POST",
+            endpoint=url,
+            log_msg_action="set incident response",
+            params=json,
+            auth_request=False,
+        )
 
-        self._request('POST', endpoint=url, log_msg_action='set incident response', params=json, auth_request=False)
-        
     def get_incident_response(self, id):
         """Get status of incident response for one incident."""
 
+        if not self._user:
+            self._get_userid()
+
         url = FSR_ENDPOINT_INCIDENTS.format(id)
 
-        response = self._request('GET', endpoint=url, log_msg_action='get incident response', auth_request=False)
+        response = self._request(
+            "GET",
+            endpoint=url,
+            log_msg_action="get incident response",
+            auth_request=False,
+        )
 
-        for r in response['incident_responses']:
-           if self._user['id'] == r['user_id']:
-              return r
+        for r in response["incident_responses"]:
+            if self._user["id"] == r["user_id"]:
+                return r
 
         return None
 
@@ -152,36 +205,52 @@ class FireServiceRota(object):
             self._get_userid()
 
         today = datetime.datetime.now().astimezone()
-        tomorrow = today + datetime.timedelta(days = 1)
+        tomorrow = today + datetime.timedelta(days=1)
 
-        id =  self._user['memberships'][0]['id']
-        url = FSR_ENDPOINT_MEMBERSHIPS.format(id, today.strftime("%Y-%m-%dT00:00:00%z"), tomorrow.strftime("%Y-%m-%dT00:00:00%z"))
+        id = self._user["memberships"][0]["id"]
+        url = FSR_ENDPOINT_MEMBERSHIPS.format(
+            id,
+            today.strftime("%Y-%m-%dT00:00:00%z"),
+            tomorrow.strftime("%Y-%m-%dT00:00:00%z"),
+        )
+        url = FSR_ENDPOINT_MEMBERSHIPS.format(
+            id,
+            today.strftime("%Y-%m-%dT00:00:00"),
+            tomorrow.strftime("%Y-%m-%dT00:00:00"),
+        )
 
-        response = self._request('GET', endpoint=url, log_msg_action='get memberships',
-                 auth_request=False)
+        response = self._request(
+            "GET", endpoint=url, log_msg_action="get memberships", auth_request=False
+        )
 
         if response:
-            for interval in response['intervals']:
-                if interval['available'] == True:
+            for interval in response["intervals"]:
+                if interval["available"] == True:
                     now = datetime.datetime.now().astimezone()
-                    if now > datetime.datetime.strptime(interval['start_time'], "%Y-%m-%dT%H:%M:%S.%f%z") and now < datetime.datetime.strptime(interval['end_time'], "%Y-%m-%dT%H:%M:%S.%f%z"):
-                        if 'standby_duty' in interval['detailed_availability']:
-                            interval['type'] = 'standby_duty'
-                        elif 'exception' in interval['detailed_availability']:
-                            interval['type'] = 'exception'
-                        elif 'recurring' in interval['detailed_availability']:
-                            interval['type'] = 'recurring'
+                    if now > datetime.datetime.strptime(
+                        interval["start_time"], "%Y-%m-%dT%H:%M:%S.%f%z"
+                    ) and now < datetime.datetime.strptime(
+                        interval["end_time"], "%Y-%m-%dT%H:%M:%S.%f%z"
+                    ):
+                        if "standby_duty" in interval["detailed_availability"]:
+                            interval["type"] = "standby_duty"
+                        elif "exception" in interval["detailed_availability"]:
+                            interval["type"] = "exception"
+                        elif "recurring" in interval["detailed_availability"]:
+                            interval["type"] = "recurring"
                         else:
-                            interval['type'] = 'unknown'
+                            interval["type"] = "unknown"
 
-                        if interval['assigned_function_ids']:
-                            for func in interval['assigned_function_ids']:
-                                interval['assigned_function'] = self.get_standby_function(func)['name']
+                        if interval["assigned_function_ids"]:
+                            for func in interval["assigned_function_ids"]:
+                                interval[
+                                    "assigned_function"
+                                ] = self.get_standby_function(func)["name"]
 
                         return interval
 
         interval = {}
-        interval['available'] = False
+        interval["available"] = False
         return interval
 
     def _request(
@@ -211,17 +280,21 @@ class FireServiceRota(object):
 
         try:
             response = requests.request(
-                method, url, headers=headers, params=params, json=body, timeout=FSR_DEFAULT_TIMEOUT
+                method,
+                url,
+                headers=headers,
+                params=params,
+                json=body,
+                timeout=FSR_DEFAULT_TIMEOUT,
             )
 
             try:
                 log_msg = response.json()
             except:
                 log_msg = response.text
-            _LOGGER.debug(
-                f"Request response: {response.status_code}: {log_msg}"
-            )
-            
+
+            _LOGGER.debug(f"Request response: {response.status_code}: {log_msg}")
+
             response.raise_for_status()
             return response.json()
         except HTTPError:
@@ -246,15 +319,15 @@ class FireServiceRota(object):
                     )
             elif response.status_code == 401:
                 error = json_payload.get("error")
-                if error == 'token_invalid':
+                if error == "token_invalid":
                     raise InvalidTokenError(
                         "Access token invalid; re-authentication required"
                     )
-                elif error == 'token_revoked':
+                elif error == "token_revoked":
                     raise ExpiredTokenError(
                         "Access token revoked; token refresh required"
                     )
-                elif error == 'token_expired':
+                elif error == "token_expired":
                     raise ExpiredTokenError(
                         "Access token expired; token refresh required"
                     )
@@ -278,72 +351,115 @@ class FireServiceRota(object):
                 f"Error connecting while attempting to {log_msg_action}. "
                 f"{response.status_code}: {json_payload}"
             )
+
         return None
 
 
-class FireServiceRotaIncidents(Thread, websocket.WebSocketApp):
+class FireServiceRotaIncidents:
 
-    def __init__(self, url,
-                 on_incident=None,
-                 on_close=None,
-                 on_error=None):
+    is_running = True
+
+    def __init__(self, on_incident=None):
         """
-        :param url: websocket url
         :param on_incident: function that get's called on received incident
-        :param on_error: function that get's called on error incident
         """
-        self._url = url
-        self.on_error = on_error
-        self.on_close = on_close
-
-        Thread.__init__(self)
-        websocket.WebSocketApp.__init__(self, url,
-                                        on_open=self.on_open,
-                                        on_error=self.on_error,
-                                        on_message=self.on_message,
-                                        on_close=self.on_close)
-
         self.on_incident = on_incident
+        self.ws = None
+
+    def start(self, url):
+        self._url = url
         self._recent_incidents = deque(maxlen=30)
 
-    def on_open(self):
-        _LOGGER.debug('Websocket open')
+        _LOGGER.debug("Websocket client start")
 
-    def on_message(self, message):
-        _LOGGER.debug('Websocket data:' + message)
+        self.ws = websocket.WebSocketApp(
+            self._url,
+            on_open=self.__on_open,
+            on_error=self.__on_error,
+            on_message=self.__on_message,
+            on_close=self.__on_close,
+        )
+
+        self.wst = threading.Thread(target=lambda: self.ws.run_forever())
+        self.wst.daemon = True
+        self.wst.start()
+
+    def stop(self):
+        """
+        close websocket
+        """
+        self.is_running = False
+        self.ws.close()
+        _LOGGER.debug("Websocket client stopped")
+
+    def __on_open(self):
+        _LOGGER.debug("Websocket open")
+
+    def __on_close(self):
+        """
+        On Close Listener
+        """
+        if self.is_running:
+            _LOGGER.info("Websocket restart")
+
+            self.ws = websocket.WebSocketApp(
+                self._url,
+                on_open=self.__on_open,
+                on_error=self.__on_error,
+                on_message=self.__on_message,
+                on_close=self.__on_close,
+            )
+            self.wst = threading.Thread(target=lambda: self.ws.run_forever())
+            self.wst.daemon = True
+            self.wst.start()
+
+    def __on_message(self, message):
+        _LOGGER.debug("Websocket data:" + message)
         try:
             message = json.loads(message)
             if "type" not in message:
-                if "identifier" in message and json.loads(message["identifier"])["channel"] == "IncidentNotificationsChannel":
+                if (
+                    "identifier" in message
+                    and json.loads(message["identifier"])["channel"]
+                    == "IncidentNotificationsChannel"
+                ):
                     incident = message["message"]
                     """mark new and update messages"""
-#                   if "address" in incident and "message_to_speech_url" in incident:
                     incident_id = incident["id"]
                     if incident_id not in self._recent_incidents:
                         self._recent_incidents.append(incident_id)
-                        incident['trigger'] = 'new'
+                        incident["trigger"] = "new"
                         _LOGGER.debug("New incident received")
                     else:
-                        incident['trigger'] = 'update'
+                        incident["trigger"] = "update"
                         _LOGGER.debug("Incident update received")
                     self.on_incident(incident)
                 else:
                     _LOGGER.debug(f"Malformed data received\n{message}")
             elif message["type"] == "welcome":
                 _LOGGER.debug("Subscribing to the incidents channel")
-                self.send(json.dumps({"command": "subscribe",
-                           "identifier": json.dumps({ "channel": "IncidentNotificationsChannel" })}))
+                self.ws.send(
+                    json.dumps(
+                        {
+                            "command": "subscribe",
+                            "identifier": json.dumps(
+                                {"channel": "IncidentNotificationsChannel"}
+                            ),
+                        }
+                    )
+                )
             elif message["type"] == "confirm_subscription":
                 _LOGGER.debug("Succesfully subscribed to incidents channel")
             elif message["type"] == "ping":
-                 pass
+                pass
             else:
                 _LOGGER.debug(f"Received unknown type: {message}")
         except Exception as e:
             _LOGGER.exception(e)
 
-    def run_forever(self, sockopt=None, sslopt=None, ping_interval=0, ping_timeout=None):
-#        websocket.enableTrace(True)
-        websocket.WebSocketApp.run_forever(self, sockopt=sockopt, sslopt=sslopt, ping_interval=ping_interval,
-                                           ping_timeout=ping_timeout)
-
+    def __on_error(self, message):
+        """
+        On Error listener
+        :param message:
+        """
+        _LOGGER.debug("Websocket error: %s", message)
